/* -*- mode: c++; c-basic-offset: 3 -*- */
#ifndef RDS_FRAME_HH
#define RDS_FRAME_HH

#include <memory>   
#include <string>   
#include <vector>   

#include "general/types.hh"
#include "general/SharedPtr.hh"

#include "framecpp/FrameH.hh"

#include "framecpp/FrAdcData.hh"
#include "framecpp/FrProcData.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrVect.hh"
#include "framecpp/Time.hh"

class RDSStream;

namespace FrameAPI
{
   namespace RDS
   {
      class Options
      {
      public:
	 Options( const INT_4U start,
		  const INT_4U stop,
		  const char* type,
		  const char* compression_method,
		  const INT_2U level,
		  const bool VerifyChecksum,
		  const bool VerifyFrameChecksum,
		  const bool VerifyTimeRange,
		  const bool VerifyDataValid,
		  const INT_4U FramesPerFile,
		  const INT_4U SecondsPerFrame,
		  const bool AllowShortFrames,
		  const bool GenerateFrameChecksum,
		  const bool FillMissingDataValidArray,
		  const bool VerifyFilenameMetadata );
	 Options( const Options& Source );

	 bool AllowShortFrames( ) const;
	 INT_2U CompressionLevel( ) const;
	 FrameCPP::FrVect::compression_scheme_type	CompressionMethod( ) const;
	 bool CreateChecksumPerFrame( ) const;
	 bool FillMissingDataValidArray( ) const;
	 INT_4U FramesPerFile( ) const;
	 void FramesPerFile( INT_4U FPF );
	 INT_4U OutputTimeEnd( ) const;
	 INT_4U OutputTimeStart( ) const;
	 const std::string& OutputType( ) const;
	 INT_4U RDSLevel( ) const;
	 void RDSLevel( INT_4U Level );
	 INT_4U SecondsPerFrame( ) const;
	 void SecondsPerFrame( INT_4U SPF );
	 bool VerifyChecksum( ) const;
	 bool VerifyChecksumPerFrame( ) const;
	 bool VerifyDataValid( ) const;
	 bool VerifyFilenameMetadata( ) const;
	 bool VerifyTimeRange( ) const;

      private:
	 //: Method for compression
	 const FrameCPP::FrVect::compression_scheme_type	m_compression_method;
	 //: Level for compression
	 const INT_2U	m_compression_level;
	 //: Number of frames per file
	 INT_4U		m_frames_per_file;
	 //: If checksum for the input file should be verified
	 const bool	m_input_verify_checksum;
	 //: If input frames' checksums should be verified
	 const bool	m_input_verify_checksum_per_frame;
	 //: If input ADC channel's dataValid field should be checked
	 const bool	m_input_verify_data_valid;
	 //: Verify that the frame's name and internal metadata agree
	 const bool	m_input_verify_filename_metadata;
	 //: If input frame's filename should be validated against its time range
	 const bool	m_input_verify_time_range;
	 //: Allow generation of short final frames
	 const bool	m_output_allow_short_frames;
	 //: If the checksum should be created for each frame on output
	 const bool	m_output_checksum_per_frame;
	 //: Allow creation of missing dataValid array
	 const bool	m_output_fill_data_valid_array;
	 //: Time when to start writing data to frame file
	 const INT_4U	m_output_start;
	 //: Time when to stop writing data to frame file
	 const INT_4U	m_output_stop;
	 //: Use type of frame file name
	 std::string	m_output_type;
	 //: The current rds level
	 // Not really a command line option but is calculated from the type
	 //  field.
	 INT_4U		m_rds_level;
	 //: Number of seconds of data for each frame
	 INT_4U		m_seconds_per_frame;

	 //: The equal operator is not allowed since all data members are constant
	 const Options& operator=( const Options& RHS );

	 std::string validate_type( const char* Type );
      }; // Class - Options

      class FileOptions
	 : public Options
      {
      public:
	 FileOptions( const INT_4U Start,
		      const INT_4U Stop,
		      const char* dir,
		      const char* type,
		      const char* compression_method,
		      const INT_2U level,
		      const bool VerifyChecksum,
		      const bool VerifyFrameChecksum,
		      const bool VerifyTimeRange,
		      const bool VerifyDataValid,
		      const INT_4U FramesPerFile,
		      const INT_4U SecondsPerFrame,
		      const bool AllowShortFrames,
		      const bool GenerateFrameChecksum,
		      const bool FillMissingDataValidArray,
		      const bool VerifyFilenameMetadata,
		      const std::string& MD5SumOutputDirectory );
	 
	 FileOptions( const FileOptions& Source );

	 const std::string& OutputDirectory( ) const;

	 const std::string& MD5SumOutputDirectory( ) const;
      private:
	 //: Directory to write result frames to
	 const std::string	m_output_directory;
	 //: Where to put the md5sum calculations
	 std::string	m_md5sum_output_directory;

      };

   } // namespace - RDS
} // namespace - FrameAPI

//-----------------------------------------------------------------------------
//   
//: Base functional to create RDS frames.
//   
class RDSFrame
{
public:
   typedef std::vector< std::string > frame_file_container_type;
   typedef std::vector< std::string > channel_container_type;
   typedef ::FrameAPI::RDS::Options Options;
   
   typedef General::SharedPtr< RDSStream >	stream_type;

   RDSFrame( const char* frame_files, 
             const char* channels,
	     const Options& CommandOptions );

   RDSFrame( const frame_file_container_type& frame_files, 
             const channel_container_type& channels,
	     const Options& CommandOptions );

   virtual ~RDSFrame();
   
   INT_4U GetNumberOfChannels( ) const;
   INT_4U GetNumberOfFrameGroups( ) const;
   INT_4U GetNumberOfFrameFiles( ) const;

   void ProcessRequest( stream_type Output );

protected:
   // Useful typedefs
   typedef General::SharedPtr< FrameCPP::FrameH >	frame_h_type;
   typedef General::SharedPtr< FrameCPP::FrAdcData >	fr_adc_data_type;
   typedef General::SharedPtr< FrameCPP::FrProcData >	fr_proc_data_type;

   typedef std::vector< std::string >::const_iterator stringv_const_iterator;

   Options	m_options;

   virtual void processChannel( INT_4U ChannelOffset,
				fr_adc_data_type Adc )
      = 0;

   virtual void processChannel( INT_4U ChannelOffset,
				fr_proc_data_type Proc )
      = 0;
			
   // Close the current frame file
   virtual void writeFrameToFile( );

   // Accessors
   static FrameCPP::FrProcData*
   getProcChannel(  const std::string& name, 
		    FrameCPP::FrameH::procData_type* const proc );      
   
   // Accessors
   static const std::string& getHistoryName( );
   
   // Get some pieces out of the frame
   FrameCPP::FrameH::procData_type* getResultProcData( );

#if 0
   // Flags for object insertion.
   static const bool mAllocate;
   static const bool mOwns;
#endif /* 0 */
   
   //: Result frame
   frame_h_type		mResultFrame;

   //: Frame writer object.
   //
   // It is necessary to keep this one open while frames are being processed
   //   to minimize memory usage by writing frames as soon as they are
   //   available.
   // std::auto_ptr< FileWriter > mWriter;
   General::SharedPtr< RDSStream > m_stream;

   virtual void createHistory();   

   const std::string& getChannelName( INT_4U Offset ) const;
   
#if 0
   //--------------------------------------------------------------------
   // Routine to generate the name of the output file. If the file
   //   already exists, then throw an exception.
   //--------------------------------------------------------------------
   std::string	generateOutputName( INT_4U FileStart,
				      REAL_8 FileDeltaT ) const;
#endif /* 0 */

private:
   class private_type;

   //--------------------------------------------------------------------
   // Holder for private information
   //--------------------------------------------------------------------

   std::string	m_ifo_list;

   private_type*	m_p;

   //: No default constructor 
   RDSFrame();
   
   //: No copy constructor
   RDSFrame( const RDSFrame& );
   
   //: No assignment operator
   RDSFrame& operator=( const RDSFrame& );

#if 0
   //--------------------------------------------------------------------
   // Routine to generate the name of the temporary file. If the file
   //   already exists, then throw an exception.
   //--------------------------------------------------------------------
   std::string	generate_tmp_output_name( const INT_4U FrameStart ) const;
#endif /* 0 */

   void init( const frame_file_container_type& frame_files, 
	      const channel_container_type& channels );

};

//=======================================================================
// 
//=======================================================================
namespace FrameAPI
{
   namespace RDS
   {
      //=================================================================
      //
      //=================================================================
      inline bool Options::
      AllowShortFrames( ) const
      {
	 return m_output_allow_short_frames;
      }

      inline INT_2U Options::
      CompressionLevel( ) const
      {
	 return m_compression_level;
      }

      inline FrameCPP::FrVect::compression_scheme_type Options::
      CompressionMethod( ) const
      {
	 return m_compression_method;
      }

      inline bool Options::
      CreateChecksumPerFrame( ) const
      {
	 return m_output_checksum_per_frame;
      }

      inline bool Options::
      FillMissingDataValidArray( ) const
      {
	 return m_output_fill_data_valid_array;
      }

      inline INT_4U Options::
      FramesPerFile( ) const
      {
	 return m_frames_per_file;
      }

      inline void Options::
      FramesPerFile( INT_4U FPF )
      {
	 m_frames_per_file = FPF;
      }

      inline INT_4U Options::
      OutputTimeEnd( ) const
      {
	 return m_output_stop;
      }

      inline INT_4U Options::
      OutputTimeStart( ) const
      {
	 return m_output_start;
      }

      inline const std::string& Options::
      OutputType( ) const
      {
	 return m_output_type;
      }

      inline INT_4U Options::
      RDSLevel( ) const
      {
	 return m_rds_level;
      }

      inline void Options::
      RDSLevel( INT_4U Level )
      {
	 m_rds_level = Level;
      }

      inline INT_4U Options::
      SecondsPerFrame( ) const
      {
	 return m_seconds_per_frame;
      }

      inline void Options::
      SecondsPerFrame( INT_4U SPF )
      {
	 m_seconds_per_frame = SPF;
      }

      inline bool Options::
      VerifyChecksum( ) const
      {
	 return m_input_verify_checksum;
      }

      inline bool Options::
      VerifyChecksumPerFrame( ) const
      {
	 return m_input_verify_checksum_per_frame;
      }

      inline bool Options::
      VerifyDataValid( ) const
      {
	 return m_input_verify_data_valid;
      }

      inline bool Options::
      VerifyFilenameMetadata( ) const
      {
	 return m_input_verify_filename_metadata;
      }

      inline bool Options::
      VerifyTimeRange( ) const
      {
	 return m_input_verify_time_range;
      }

      //=================================================================
      //
      //=================================================================
      inline const std::string& FileOptions::
      MD5SumOutputDirectory( ) const
      {
	 return m_md5sum_output_directory;
      }

      inline const std::string& FileOptions::
      OutputDirectory( ) const
      {
	 return m_output_directory;
      }

   } // namespace - RDS
} // namespace - FrameAPI

//=======================================================================
// Inline methods for class RDSFrame
//=======================================================================
inline FrameCPP::FrameH::procData_type* RDSFrame::
getResultProcData( )
{
   return &( mResultFrame->RefProcData( ) );
}

#endif /* RDS_FRAME_HH */

